STAC_ASSETS_KEY = "assets"
STAC_DESCRIPTION_KEY = "description"
STAC_EXTENSIONS_KEY = "stac_extensions"
STAC_EXTENT_BBOX_KEY = "bbox"
STAC_EXTENT_KEY = "extent"
STAC_EXTENT_SPATIAL_KEY = "spatial"
STAC_EXTENT_TEMPORAL_INTERVAL_KEY = "interval"
STAC_EXTENT_TEMPORAL_KEY = "temporal"
STAC_FILE_CHECKSUM_KEY = "file:checksum"
STAC_GEOMETRY_KEY = "geometry"
STAC_HREF_KEY = "href"
STAC_ID_KEY = "id"
STAC_LICENSE_KEY = "license"
STAC_LINKS_KEY = "links"
STAC_MEDIA_TYPE_GEOJSON = "application/geo+json"
STAC_MEDIA_TYPE_JSON = "application/json"
STAC_PROPERTIES_DATETIME_KEY = "datetime"
STAC_PROPERTIES_KEY = "properties"
STAC_REL_CHILD = "child"
STAC_REL_ITEM = "item"
STAC_REL_KEY = "rel"
STAC_REL_PARENT = "parent"
STAC_REL_ROOT = "root"
STAC_REL_SELF = "self"
STAC_TITLE_KEY = "title"
STAC_TYPE_CATALOG = "Catalog"
STAC_TYPE_COLLECTION = "Collection"
STAC_TYPE_ITEM = "Feature"
STAC_TYPE_KEY = "type"
STAC_VERSION = "1.0.0"
STAC_VERSION_KEY = "stac_version"

LINZ_SCHEMA_DIRECTORY = "stac/v0.0.7/linz"
LINZ_SCHEMA_PATH = f"{LINZ_SCHEMA_DIRECTORY}/schema.json"
LINZ_STAC_EXTENSION_URL = f"https://linz.github.io/{LINZ_SCHEMA_PATH}"
LINZ_STAC_CREATED_KEY = "linz:created"
LINZ_STAC_SECURITY_CLASSIFICATION_KEY = "linz:security_classification"
LINZ_STAC_LIFECYCLE_KEY = "linz:lifecycle"
LINZ_STAC_PROVIDERS_KEY = "linz:providers"
LINZ_STAC_UPDATED_KEY = "linz:updated"

QUALITY_SCHEMA_PATH = "stac/v0.0.7/quality/schema.json"
QUALITY_LINEAGE_KEY = "quality:lineage"

VERSION_VERSION_KEY = "version"
