DB_KEY_SEPARATOR = "#"

CHECK_ID_PREFIX = f"CHECK{DB_KEY_SEPARATOR}"
DATASET_ID_PREFIX = f"DATASET{DB_KEY_SEPARATOR}"
URL_ID_PREFIX = f"URL{DB_KEY_SEPARATOR}"
VERSION_ID_PREFIX = f"VERSION{DB_KEY_SEPARATOR}"
